//
//  mmBusinessObject.swift
//  mmiOS8
//
//  Created by Kevin McNeish on 9/9/14.
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import CoreData

enum SaveStateGeneric {
    case Error, RulesBroken, SaveComplete
}

public class mmBusinessObjectGeneric
{
    var dbName : String = ""
    var entityClassName : String = ""
    var copyDatabaseIfNotPresent : Bool = false
    var error : NSError?
    
    init()
    {
    }
    
    // Create a new entity of the default type
    func createEntity() -> NSManagedObject
    {
        return NSEntityDescription.insertNewObjectForEntityForName(self.entityClassName, inManagedObjectContext: self.managedObjectContext!) as NSManagedObject
    }
    
    // Gets all entities of the default type
    func getAllEntities() -> Array<NSManagedObject>?
    {
        var error: NSError?
        
        // Create the request object
        var request : NSFetchRequest = NSFetchRequest()
        
        // Set the entity type to be fetched
        var entityDescription : NSEntityDescription! = NSEntityDescription.entityForName(self.entityClassName, inManagedObjectContext: self.managedObjectContext!)
        
        request.entity = entityDescription
        
        // Execute the fetch
        var results = self.managedObjectContext?.executeFetchRequest(request, error: &error)
        
        return results as Array<NSManagedObject>?
    }
    
    // Save changes to the specified entity
    func saveEntity(entity: NSManagedObject) -> (state: SaveState, message: String?)
    {
        var saveState: SaveState
        var saveMessage: String?
        
        // Check the business rules
        saveMessage = self.checkRulesForEntity(entity)
        
        if saveMessage == nil
        {
            let result = self.saveEntities()
            saveState = result.state
            saveMessage = result.message
        }
        else
        {
            // Business rules failed
            saveState = SaveState.RulesBroken
        }
        return (state: saveState, message: saveMessage)
    }
    
    // Saves changes to all entities
    func saveEntities() -> (state: SaveState, message: String?)
    {
        var saveState: SaveState
        var saveMessage: String?
        
        // Business rules passed
        var objectContext = self.managedObjectContext!
        
        if objectContext.hasChanges && !objectContext.save(&error)
        {
            saveState = SaveState.Error
            saveMessage = error?.description
        }
        else
        {
            saveState = SaveState.SaveComplete
            saveMessage = nil
        }
        
        return (saveState, saveMessage)
    }
    
    // Checks business rules for the specified entity
    func checkRulesForEntity(entity: NSManagedObject) -> NSString?
    {
        return nil
    }
    
    // Returns the managed object context for the application (which is already bound to the persistent store coordinator for the application.) This property is optional since there are legitimate error conditions that could cause the creation of the context to fail.
    lazy var managedObjectContext: NSManagedObjectContext? = {
        
        let coordinator = self.persistentStoreCoordinator
        if coordinator == nil {
            return nil
        }
        var managedObjectContext = NSManagedObjectContext()
        managedObjectContext.persistentStoreCoordinator = coordinator
        return managedObjectContext
        }()
    
    // The directory the application uses to store the Core Data store file. This code uses a directory named "com.oakleafsd.<ProjectName>" in the application's documents Application Support directory.
    lazy var applicationDocumentsDirectory: NSURL = {
        let urls = NSFileManager.defaultManager().URLsForDirectory(.DocumentDirectory, inDomains: .UserDomainMask)
        
        println(urls);
        
        return urls[urls.count-1] as NSURL
        }()
    
    // The managed object model for the application. This property is not optional. It is a fatal error for the application not to be able to find and load its model.
    lazy var managedObjectModel: NSManagedObjectModel = {
        
        let modelURL = NSBundle.mainBundle().URLForResource(self.dbName, withExtension: "momd")!
        var model: NSManagedObjectModel? = NSManagedObjectModel(contentsOfURL: modelURL)
        return model!
        }()
    
    // The persistent store coordinator for the application. This implementation creates and return a coordinator, having added the store for the application to it. This property is optional since there are legitimate error conditions that could cause the creation of the store to fail.
    lazy var persistentStoreCoordinator: NSPersistentStoreCoordinator? = {
        
        let url : NSURL = self.applicationDocumentsDirectory.URLByAppendingPathComponent("\(self.dbName).sqlite")
        
        if self.copyDatabaseIfNotPresent
        {
            // Get the Documents directory
            var paths = NSSearchPathForDirectoriesInDomains(NSSearchPathDirectory.DocumentDirectory, NSSearchPathDomainMask.UserDomainMask, true)
            var docsDir = paths[0] as String
            
            // Append the database name to the path
            var dbcPath = docsDir.stringByAppendingPathComponent(self.dbName + ".sqlite")
            
            var fileManager = NSFileManager.defaultManager()
            
            // Check if the sqlite database already exists
            if !fileManager.fileExistsAtPath(dbcPath)
            {
                // It doesn't. Check if a .sqlite database was distributed with this app
                var defaultStorePath = NSBundle.mainBundle().pathForResource(self.dbName, ofType: "sqlite")
                if defaultStorePath != nil
                {
                    // It was, copy the embedded database to the app's Documents directory
                    fileManager.copyItemAtPath(defaultStorePath!, toPath: dbcPath, error: nil)
                }
            }
        }
        
        var error: NSError? = nil
        // Create the coordinator and store
        var coordinator: NSPersistentStoreCoordinator? = NSPersistentStoreCoordinator(managedObjectModel: self.managedObjectModel)
        var failureReason = "There was an error creating or loading the application's saved data."
        if coordinator!.addPersistentStoreWithType(NSSQLiteStoreType, configuration: nil, URL: url, options: nil, error: &error) == nil {
            coordinator = nil
            // Report any error we got.
            let dict = NSMutableDictionary()
            dict[NSLocalizedDescriptionKey] = "Failed to initialize the application's saved data"
            dict[NSLocalizedFailureReasonErrorKey] = failureReason
            dict[NSUnderlyingErrorKey] = error
            error = NSError(domain: "YOUR_ERROR_DOMAIN", code: 9999, userInfo: dict)
            
            if error?.code == 134100
            {
                self.performAutomaticLightweightMigration()
            }
            NSLog("Unresolved error \(error), \(error!.userInfo)")
            abort()
        }
        
        return coordinator
        }()
    
    // Mark the specified entity for deletion
    func deleteEntity(entity: NSManagedObject)
    {
        self.managedObjectContext?.deleteObject(entity)
    }
    
    // Register a related business controller object
    // This causes them to use the same object context
    func registerChildObject(controllerObject: mmBusinessObject)
    {
        controllerObject.managedObjectContext = self.managedObjectContext
    }
    
    // Migrates the database to the structure of a new core data model
    func performAutomaticLightweightMigration()
    {
        var error: NSError?
        
        let storeURL = self.applicationDocumentsDirectory.URLByAppendingPathComponent("/(self.dbName).sqlite")
        
        var options : Dictionary<NSNumber, NSString> =
        [NSNumber(bool: true) : NSMigratePersistentStoresAutomaticallyOption,
            NSNumber(bool: true) : NSInferMappingModelAutomaticallyOption]
        
        var store = persistentStoreCoordinator!.addPersistentStoreWithType(NSSQLiteStoreType, configuration: nil, URL: storeURL, options: options, error: &error)
        
        if store == nil
        {
            println("Error performing lightweight migration:\(error?.description)")
        }
    }

}
